@@ -24,10 +24,10 @@ oai <- function(var, cols, area) {
 }
 
 jenks.tests <- function(clI, area) {
-   if (class(clI) != "classIntervals") stop("Class interval object required")
+   if (!inherits(clI, "classIntervals")) stop("Class interval object required")
    cols <- findCols(clI)
-   res <- c("# classes"=length(clI$brks)-1, 
-     "Goodness of fit"=gvf(clI$var, cols), 
+   res <- c("# classes"=length(clI$brks)-1,
+     "Goodness of fit"=gvf(clI$var, cols),
      "Tabular accuracy"=tai(clI$var, cols))
    if (!missing(area)) {
       if (length(area) != length(cols))
@@ -38,7 +38,7 @@ jenks.tests <- function(clI, area) {
 }
 
 plot.classIntervals <- function(x, pal, ...) {
-   if (class(x) != "classIntervals") stop("Class interval object required")
+   if (!inherits(x, "classIntervals")) stop("Class interval object required")
    if (length(pal) < 2) stop("pal must contain at least two colours")
    pal_out <- colorRampPalette(pal)(length(x$brks)-1)
    plot(ecdf(x$var), ...)
@@ -104,18 +104,21 @@ classIntervals <- function(var, n, style="quantile", rtimes=3, ..., intervalClos
   }
   nobs <- length(unique(var))
   if (nobs == 1) stop("single unique value")
+  # Fix 22: Diego Hernangómez
+  needn <- !(style %in% c("dpih", "headtails", "box"))
+
   if (missing(n)) n <- nclass.Sturges(var)
-  if (n < 2) stop("n less than 2")
+  if (n < 2 & needn) stop("n less than 2")
   n <- as.integer(n)
   pars <- NULL
-  if (n > nobs) {
+  if (n > nobs & needn) {
     if (warnSmallN) {
       warning(paste("n greater than number of different finite values",
       "n reset to number of different finite values", sep="\\n"))
     }
     n <- nobs
   }
-  if (n == nobs) {
+  if (n == nobs & needn) {
     if (warnSmallN) {
       warning(paste("n same as number of different finite values",
       "each different finite value is a separate class", sep="\\n"))
@@ -128,7 +131,7 @@ classIntervals <- function(var, n, style="quantile", rtimes=3, ..., intervalClos
   } else {
 # introduced related to https://github.com/r-spatial/classInt/issues/7
     sampling <- FALSE
-    if (warnLargeN && 
+    if (warnLargeN &&
       (style %in% c("kmeans", "hclust", "bclust", "fisher", "jenks"))) {
       if (nobs > largeN) {
         warning("N is large, and some styles will run very slowly; sampling imposed")
@@ -142,7 +145,7 @@ classIntervals <- function(var, n, style="quantile", rtimes=3, ..., intervalClos
 # Matthieu Stigler 111110
       dots <- list(...)
       fixedBreaks <- sort(dots$fixedBreaks)
-      if (is.null(fixedBreaks)) 
+      if (is.null(fixedBreaks))
         stop("fixed method requires fixedBreaks argument")
 #      if (length(fixedBreaks) != (n+1))
 #        stop("mismatch between fixedBreaks and n")
@@ -157,7 +160,7 @@ classIntervals <- function(var, n, style="quantile", rtimes=3, ..., intervalClos
         }
       }
       if (any(diff(fixedBreaks) < 0)) stop("decreasing fixedBreaks found")
-      if (min(var) < fixedBreaks[1] || 
+      if (min(var) < fixedBreaks[1] ||
         max(var) > fixedBreaks[length(fixedBreaks)])
           warning("variable range greater than fixedBreaks")
       brks <- fixedBreaks
@@ -178,11 +181,11 @@ classIntervals <- function(var, n, style="quantile", rtimes=3, ..., intervalClos
     } else if (style =="kmeans") {
 # stats
       pars <- try(kmeans(x=var, centers=n, ...))
-      if (class(pars) == "try-error") {
+      if (inherits(pars, "try-error")) {
         warning("jittering in kmeans")
         jvar <- jitter(rep(x=var, times=rtimes))
         pars <- try(kmeans(x=jvar, centers=n, ...))
-	if (class(pars) == "try-error") stop("kmeans failed after jittering")
+	if (inherits(pars, "try-error")) stop("kmeans failed after jittering")
         else {
           cols <- match(pars$cluster, order(c(pars$centers)))
           rbrks <- unlist(tapply(jvar, factor(cols), range))
@@ -205,11 +208,11 @@ classIntervals <- function(var, n, style="quantile", rtimes=3, ..., intervalClos
     } else if (style =="bclust") {
 # e1071, class
       pars <- try(bclust(x=var, centers=n, ...))
-      if (class(pars) == "try-error") {
+      if (inherits(pars, "try-error")) {
         warning("jittering in bclust")
         jvar <- jitter(rep(x=var, times=rtimes))
         pars <- try(bclust(x=jvar, centers=n, ...))
-	if (class(pars) == "try-error") stop("bclust failed after jittering")
+	if (inherits(pars, "try-error")) stop("bclust failed after jittering")
         else {
           cols <- match(pars$cluster, order(c(pars$centers)))
           rbrks <- unlist(tapply(jvar, factor(cols), range))
@@ -223,7 +226,7 @@ classIntervals <- function(var, n, style="quantile", rtimes=3, ..., intervalClos
     } else if (style =="fisher") {
 # introduced related to https://github.com/r-spatial/classInt/issues/7
       if (sampling) {
-        pars <- fish(x=sample(x=var, size=nsamp), k=n)
+        pars <- fish(x=c(range(var), sample(x=var, size=nsamp)), k=n)
       } else {
         pars <- fish(x=var, k=n)
       }
@@ -240,7 +243,7 @@ classIntervals <- function(var, n, style="quantile", rtimes=3, ..., intervalClos
 # introduced related to https://github.com/r-spatial/classInt/issues/7
            if (sampling) {
              message("Use \"fisher\" instead of \"jenks\" for larger data sets")
-             d <- sort(sample(x=var, size=nsamp))
+             d <- sort(c(range(var), sample(x=var, size=nsamp)))
            } else {
              d <- sort(var)
            }
@@ -285,7 +288,7 @@ classIntervals <- function(var, n, style="quantile", rtimes=3, ..., intervalClos
              k <- id #lower
              last <- k -1 #upper
            }
-# change uncontributed by Richard Dunlap 090512           
+# change uncontributed by Richard Dunlap 090512
 # with the specification of intervalClosure for the presentation layer,
 # don't need to change this
            brks<-d[c(1, kclass)]
@@ -302,7 +305,87 @@ classIntervals <- function(var, n, style="quantile", rtimes=3, ..., intervalClos
                vmax <- max(var)
            }
            brks <- seq(vmin, vmax, by=h)
-      
+      } else if (style == "headtails") {
+             # Contributed Diego Hernangómez
+# https://github.com/mtennekes/tmap/issues/555
+             dots <- list(...)
+             thr <- ifelse(is.null(dots$thr),
+                           .4,
+                           dots$thr)
+
+             thr <-  min(1,max(0, thr))
+             head <- var
+             breaks <- min(var, na.rm = TRUE) #Init with minimum
+             for (i in 1:100) {
+               mu <- mean(head, na.rm = TRUE)
+               breaks <- c(breaks, mu)
+               ntot <- length(head)
+               #Switch head
+               head <- head[head > mu]
+               prop <- length(head) / ntot
+               keepiter <- prop <= thr & length(head) > 1
+               if (isFALSE(keepiter)) {break}
+             }
+             #Add max to complete intervals
+             brks <- sort(unique(c(breaks,
+                                   max(var, na.rm = TRUE))))
+      } else if (style == "maximum") {
+
+        # 2022-06-05 Josiah Parry
+        x_sort <- sort(var)
+        diffs <- diff(x_sort)
+
+        n_breaks <- sort(diffs, decreasing = TRUE)[1:(n-1)]
+
+        # identify values to average for breaks
+        int_end_index <- which(diffs %in% n_breaks)
+        int_nb_index <- which(diffs %in% n_breaks) + 1
+
+        m <- matrix(
+          c(x_sort[int_nb_index], x_sort[int_end_index]),
+          ncol = 2
+        )
+
+        brks <- c(min(x_sort), rowSums(m) / 2, max(x_sort))
+
+
+      } else if (style == "box"){
+        # 2022-09-22 Diego Hernangomez, see:
+        # https://github.com/r-spatial/classInt/issues/18
+        # Adapted from:
+        # https://spatialanalysis.github.io/lab_tutorials/4_R_Mapping.html#box-map
+        
+        dots <- list(...)
+        iqr_mult <- ifelse(is.null(dots$iqr_mult), 1.5, dots$iqr_mult)
+        qtype <- ifelse(is.null(dots$type), 7, dots$type)
+        
+        qv <- unname(quantile(var, type=qtype))
+        iqr <- iqr_mult * (qv[4] - qv[2])
+        upfence <- qv[4] + iqr
+        lofence <- qv[2] - iqr
+        
+        # initialize break points vector
+        bb <- vector(mode="numeric",length=7)
+        
+        # logic for lower and upper fences
+        if (lofence < qv[1]) {  # no lower outliers
+          bb[1] <- lofence
+          bb[2] <- floor(qv[1])
+        } else {
+          bb[2] <- lofence
+          bb[1] <- qv[1]
+        }
+        if (upfence > qv[5]) { # no upper outliers
+          bb[7] <- upfence
+          bb[6] <- ceiling(qv[5])
+        } else {
+          bb[6] <- upfence
+          bb[7] <- qv[5]
+        }
+        bb[3:5] <- qv[2:4]
+        
+        brks <- bb
+        
       } else stop(paste(style, "unknown"))
   }
   if (is.null(brks)) stop("Null breaks")
@@ -334,7 +417,7 @@ classIntervals <- function(var, n, style="quantile", rtimes=3, ..., intervalClos
   brks <- c(rbrks[1], rbrks[nb])
   if (nb > 2) {
     if (nb == 3) brks <- append(brks, rbrks[2], 1)
-    else { 
+    else {
       ins <- NULL
       for (i in as.integer(seq(2,(nb-2),2))) {
         ins <- c(ins, ((rbrks[i]+rbrks[i+1])/2))
@@ -347,7 +430,7 @@ classIntervals <- function(var, n, style="quantile", rtimes=3, ..., intervalClos
 
 findColours <- function(clI, pal, under="under", over="over", between="-",
   digits = getOption("digits"), cutlabels=TRUE) {
-  if (class(clI) != "classIntervals") stop("Class interval object required")
+  if (!inherits(clI, "classIntervals")) stop("Class interval object required")
   if (is.null(clI$brks)) stop("Null breaks")
   if (length(pal) < 2) stop("pal must contain at least two colours")
   cols <- findCols(clI)
@@ -365,15 +448,19 @@ findColours <- function(clI, pal, under="under", over="over", between="-",
 # change contributed by Richard Dunlap 090512
 # Looks for intervalClosure attribute to allow specification of
 # whether partition intervals are closed on the left or the right
-findCols <- function(clI)  {
-  if (class(clI) != "classIntervals") stop("Class interval object required")
+findCols <- function(clI, factor = FALSE)  {
+  if (!inherits(clI, "classIntervals")) stop("Class interval object required")
   if (is.null(clI$brks)) stop("Null breaks")
   if (is.null(attr(clI, "intervalClosure")) || (attr(clI, "intervalClosure") == "left")) {
   	cols <- findInterval(clI$var, clI$brks, all.inside=TRUE)
   }
   else {
-	cols <- apply(array(apply(outer(clI$var, clI$brks, ">"), 1, sum)), 1, max, 1)  	
-  }  
+	cols <- apply(array(apply(outer(clI$var, clI$brks, ">"), 1, sum)), 1, max, 1)
+  }
+  if (factor) {
+    col_vals <- names(tableClassIntervals(cols, clI$brks))
+    cols <- factor(cols, levels = 1:length(col_vals), labels = col_vals)
+  }
   cols
 }
 
@@ -393,7 +480,7 @@ tableClassIntervals <- function(cols, brks, under="under", over="over",
       sep <- ""
       between=","
    }
-   
+
    if (is.null(intervalClosure) || (intervalClosure=="left")) {
    	left = "["
    	right = ")"
@@ -401,8 +488,8 @@ tableClassIntervals <- function(cols, brks, under="under", over="over",
    else {
    	left = "("
    	right = "]"
-   }   
-   
+   }
+
 #The two global endpoints are going through roundEndpoint to get
 # formatting right, nothing more
    if (cutlabels) nres[1] <- paste("[", roundEndpoint(brks[1], intervalClosure, dataPrecision), between, roundEndpoint(brks[2], intervalClosure, dataPrecision), right, sep=sep)
@@ -421,7 +508,7 @@ tableClassIntervals <- function(cols, brks, under="under", over="over",
 # Matthieu Stigler 120705 unique
    ## Assign unique label for intervals containing same left-right points
   if(unique&!missing(var)){
-  
+
     tab_unique<-tapply(var, cols, function(x) length(unique(x)))
 #    tab_unique_vals<-tapply(var, cols, function(x) length(unique(x)))
     if(any(tab_unique==1)){
@@ -433,7 +520,7 @@ tableClassIntervals <- function(cols, brks, under="under", over="over",
       names(tab)[w.unique] <- tapply(var[cols.unique ], cols[cols.unique ], function(x) if(is.null(dataPrecision)) unique(x) else round(unique(x), dataPrecision))
     }
   }
-  
+
    tab
 }
 
@@ -446,21 +533,21 @@ roundEndpoint <- function(x, intervalClosure=c("left", "right"), dataPrecision)
       retval <- x
    }
    else if (is.null(intervalClosure) || (intervalClosure=="left")) {
-      retval <- ceiling(x * 10^dataPrecision) / 10^dataPrecision   
+      retval <- ceiling(x * 10^dataPrecision) / 10^dataPrecision
    }
    else
    {
-      retval <- floor(x * 10^dataPrecision) / 10^dataPrecision      
+      retval <- floor(x * 10^dataPrecision) / 10^dataPrecision
    }
-   digits = getOption("digits")   
-   format(retval, digits=digits, trim=TRUE)   
+   digits = getOption("digits")
+   format(retval, digits=digits, trim=TRUE)
 } #FIXME output trailing zeros in decimals
 
 print.classIntervals <- function(x, digits = getOption("digits"), ..., under="under", over="over", between="-", cutlabels=TRUE, unique=FALSE) {
-   if (class(x) != "classIntervals") stop("Class interval object required")
+   if (!inherits(x, "classIntervals")) stop("Class interval object required")
    cat("style: ", attr(x, "style"), "\n", sep="")
    UNITS <- attr(x, "var_units")
-   if (is.null(UNITS)) UNITS <- "" 
+   if (is.null(UNITS)) UNITS <- ""
    else UNITS <- paste0(UNITS, " ")
    nP <- nPartitions(x)
    if (is.finite(nP)) cat("  one of ", prettyNum(nP, big.mark = ","),
@@ -490,9 +577,9 @@ nPartitions <- function(x) {
 }
 
 getBclustClassIntervals <- function(clI, k) {
-  if (class(clI) != "classIntervals") stop("Class interval object required")
+  if (!inherits(clI, "classIntervals")) stop("Class interval object required")
   if (missing(k)) k <- length(clI$brks)-1
-  if (class(attr(clI, "parameters")) != "bclust") 
+  if (!inherits(attr(clI, "parameters"), "bclust"))
     stop("Class interval object not made with style=\"bclust\"")
 
   ovar <- clI$var
@@ -518,9 +605,9 @@ getBclustClassIntervals <- function(clI, k) {
 }
 
 getHclustClassIntervals <- function(clI, k) {
-  if (class(clI) != "classIntervals") stop("Class interval object required")
+  if (!inherits(clI, "classIntervals")) stop("Class interval object required")
   if (missing(k)) k <- length(clI$brks)-1
-  if (class(attr(clI, "parameters")) != "hclust") 
+  if (!inherits(attr(clI, "parameters"), "hclust"))
     stop("Class interval object not made with style=\"hclust\"")
 
   ovar <- clI$var
